package com.tzld.piaoquan.risk.control.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.risk.control.common.enums.ExceptionEnum;
import com.tzld.piaoquan.risk.control.common.exception.CommonException;
import com.tzld.piaoquan.risk.control.config.LoginUserContext;
import com.tzld.piaoquan.risk.control.dao.mapper.UserBaseMapper;
import com.tzld.piaoquan.risk.control.dao.mapper.WorkWechatRoomInfoMapper;
import com.tzld.piaoquan.risk.control.model.dto.Page;
import com.tzld.piaoquan.risk.control.model.dto.WorkWechatRoomDTO;
import com.tzld.piaoquan.risk.control.model.dto.WorkWechatUserInfo;
import com.tzld.piaoquan.risk.control.model.po.UserBase;
import com.tzld.piaoquan.risk.control.model.po.UserBaseExample;
import com.tzld.piaoquan.risk.control.model.po.WorkWechatRoomInfo;
import com.tzld.piaoquan.risk.control.model.po.WorkWechatRoomInfoExample;
import com.tzld.piaoquan.risk.control.service.WorkWechatService;
import com.tzld.piaoquan.risk.control.util.HttpClientUtil;
import com.tzld.piaoquan.risk.control.util.HttpPoolClient;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class WorkWechatServiceImpl implements WorkWechatService {

    @Autowired
    private UserBaseMapper userBaseMapper;
    @Autowired
    private WorkWechatRoomInfoMapper workWechatRoomInfoMapper;

    private static final HttpPoolClient httpPoolClientDefault = HttpClientUtil.create(10000, 10000, 2000, 5000, 5, 10000);

    @Value("${work.wechat.tool.base.url:http://47.110.132.222:8083}")
    private String workWechatToolBaseUrl;

    @Override
    public Page<WorkWechatUserInfo> findUserInfoList(int pageSize, int currentPage, String phone) {

        Page<WorkWechatUserInfo> page = new Page<>();

        UserBaseExample userBaseExample = new UserBaseExample();
        UserBaseExample.Criteria criteria = userBaseExample.createCriteria().
                andCreateUserEqualTo(LoginUserContext.getUserId()).andVidIsNotNull();
        if (StringUtils.isNotBlank(phone)) {
            criteria.andMobileEqualTo(phone);
        }


        long total = userBaseMapper.countByExample(userBaseExample);

        userBaseExample.setPage(new Page<UserBase>(currentPage, pageSize));
        List<UserBase> userBases = userBaseMapper.selectByExample(userBaseExample);

        List<WorkWechatUserInfo> workWechatUserInfos = this.getWorkWechatUserInfos(userBases);

        page.setTotalSize((int) total);
        page.setObjs(workWechatUserInfos);
        page.setCurrentPage(currentPage);
        page.setPageSize(pageSize);
        return page;
    }

    @Override
    public Page<WorkWechatRoomDTO> findChatRoomList(String uuid, int pageSize, int currentPage, String chatRoomName) {
        Page<WorkWechatRoomDTO> page = new Page<>();

        JSONObject chatRoomInfo = this.getChatRoomInfo(uuid, 0);
        Map<String, Integer> roomAutoRemoveUserStatusMap = this.getRoomAutoRemoveUserStatusMap();

        JSONArray roomList = chatRoomInfo.getJSONArray("roomList");

        // 过滤聊天室名称（如果提供了chatRoomName参数）
        List<WorkWechatRoomDTO> filteredRooms = new ArrayList<>();
        for (int i = 0; i < roomList.size(); i++) {
            JSONObject room = roomList.getJSONObject(i);
            if (StringUtils.isNotEmpty(chatRoomName) && !StringUtils.contains(room.getString("nickname"), chatRoomName)) {
                continue;
            }

            String roomId = room.getString("room_id");

            WorkWechatRoomDTO workWechatRoomDTO = new WorkWechatRoomDTO();
            workWechatRoomDTO.setUuid(uuid);
            workWechatRoomDTO.setRoomId(roomId);
            workWechatRoomDTO.setRoomName(room.getString("nickname"));
            workWechatRoomDTO.setAutoRemoveUserSwitch(roomAutoRemoveUserStatusMap.getOrDefault(roomId, 0));
            filteredRooms.add(workWechatRoomDTO);
        }


        // 重新计算分页位置（因为列表可能经过过滤）
        int start = (currentPage - 1) * pageSize;
        int end = Math.min(start + pageSize, filteredRooms.size());

        // 确保start不越界
        start = Math.min(start, filteredRooms.size());

        // 获取当前页的数据
        List<WorkWechatRoomDTO> pageData = new ArrayList<>();
        if (start < filteredRooms.size()) {
            pageData = filteredRooms.subList(start, end);
        }

        // 设置分页数据
        page.setTotalSize(filteredRooms.size());
        page.setCurrentPage(currentPage);
        page.setPageSize(pageSize);
        page.setObjs(pageData);

        return page;
    }

    @Override
    public void updateAutoRemoveUserStatus(WorkWechatRoomDTO param) {

        if (StringUtils.isAnyBlank(param.getRoomId(), param.getUuid()) || Objects.isNull(param.getAutoRemoveUserSwitch())) {
            throw new CommonException(ExceptionEnum.PARAMS_INVALID);
        }

        Long userId = LoginUserContext.getUserId();

        WorkWechatRoomInfoExample example = new WorkWechatRoomInfoExample();
        example.createCriteria().andRoomIdEqualTo(param.getRoomId());
        List<WorkWechatRoomInfo> workWechatRoomInfos = workWechatRoomInfoMapper.selectByExample(example);
        if (CollectionUtils.isNotEmpty(workWechatRoomInfos)) {
            WorkWechatRoomInfo workWechatRoomInfo = workWechatRoomInfos.get(0);
            workWechatRoomInfo.setAutoRemoveUserSwitch(param.getAutoRemoveUserSwitch());
            workWechatRoomInfo.setUpdateUser(userId);
            workWechatRoomInfoMapper.updateByPrimaryKeySelective(workWechatRoomInfo);
        } else {
            WorkWechatRoomInfo workWechatRoomInfo = new WorkWechatRoomInfo();
            workWechatRoomInfo.setUuid(param.getUuid());
            workWechatRoomInfo.setRoomId(param.getRoomId());
            workWechatRoomInfo.setCreateUser(userId);
            workWechatRoomInfo.setUpdateUser(userId);
            workWechatRoomInfo.setAutoRemoveUserSwitch(param.getAutoRemoveUserSwitch());
            workWechatRoomInfoMapper.insertSelective(workWechatRoomInfo);
        }
    }

    @Override
    public boolean checkAutoRemoveUserStatusIsOpen(String roomId) {
        if (StringUtils.isBlank(roomId)) {
            return false;
        }
        WorkWechatRoomInfoExample example = new WorkWechatRoomInfoExample();
        example.createCriteria().andRoomIdEqualTo(roomId);
        List<WorkWechatRoomInfo> workWechatRoomInfos = workWechatRoomInfoMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(workWechatRoomInfos)) {
            return false;
        }
        return workWechatRoomInfos.get(0).getAutoRemoveUserSwitch() == 1;
    }

    private List<WorkWechatUserInfo> getWorkWechatUserInfos(List<UserBase> userBases) {
        List<WorkWechatUserInfo> workWechatUserInfos = new ArrayList<>();
        for (UserBase userBase : userBases) {
            WorkWechatUserInfo workWechatUserInfo = new WorkWechatUserInfo();
            workWechatUserInfo.setId(userBase.getId());
            workWechatUserInfo.setUuid(userBase.getUuid());
            workWechatUserInfo.setVid(userBase.getVid());
            workWechatUserInfo.setPhone(userBase.getMobile());
            workWechatUserInfo.setName(userBase.getRealname());
            workWechatUserInfo.setAvatar(userBase.getAvatar());
            workWechatUserInfo.setCorpId(userBase.getCorpId());
            workWechatUserInfo.setCorpName(userBase.getCorpName());
            workWechatUserInfo.setUnionId(userBase.getUnionid());
            workWechatUserInfo.setLoginStatus(userBase.getLoginStatus());
            workWechatUserInfos.add(workWechatUserInfo);
        }
        return workWechatUserInfos;
    }

    private JSONObject getChatRoomInfo(String uuid, int starIndex) {
        String url = workWechatToolBaseUrl + "/wxwork/GetChatroomMembers";
        JSONObject paramJson = new JSONObject();
        paramJson.put("uuid", uuid);
        paramJson.put("limit", 500);
        paramJson.put("star_index", starIndex);

        Optional<String> responseOpt = httpPoolClientDefault.postJson(url, paramJson.toJSONString());

        if (!responseOpt.isPresent()) {
            throw new CommonException(ExceptionEnum.SYSTEM_ERROR, "获取企微客户群列表异常");
        }
        JSONObject responseJson = JSON.parseObject(responseOpt.get());
        if (responseJson.getIntValue("errcode") != 0) {
            log.error("获取企微客户群列表异常. uuid: {}, errmsg: {}", uuid, responseJson.getIntValue("errmsg"));
            throw new CommonException(ExceptionEnum.SYSTEM_ERROR, "获取企微客户群列表异常");
        }
        return responseJson.getJSONObject("data");
    }

    private Map<String, Integer> getRoomAutoRemoveUserStatusMap() {
        WorkWechatRoomInfoExample example = new WorkWechatRoomInfoExample();
        List<WorkWechatRoomInfo> workWechatRoomInfos = workWechatRoomInfoMapper.selectByExample(example);
        return workWechatRoomInfos.stream().collect(Collectors.toMap(WorkWechatRoomInfo::getRoomId, WorkWechatRoomInfo::getAutoRemoveUserSwitch, (o1, o2) -> o2));
    }
}
