package com.tzld.piaoquan.risk.control.service.rule;

import java.util.ArrayList;
import java.util.List;

/**
 * 词法分析
 *
 * @author supeng
 */
public class Lexer {
    private final String input;
    private int pos = 0;

    public Lexer(String input) {
        this.input = input.trim();
    }

    public List<Token> tokenize() {
        List<Token> tokens = new ArrayList<>();
        while (pos < input.length()) {
            char c = input.charAt(pos);
            if (Character.isWhitespace(c)) {
                pos++;
            } else if (c == '(') {
                tokens.add(new Token(TokenType.LPAREN, "("));
                pos++;
            } else if (c == ')') {
                tokens.add(new Token(TokenType.RPAREN, ")"));
                pos++;
            } else if (c == '&' && pos + 1 < input.length() && input.charAt(pos + 1) == '&') {
                tokens.add(new Token(TokenType.AND, "&&"));
                pos += 2;
            } else if (c == '|' && pos + 1 < input.length() && input.charAt(pos + 1) == '|') {
                tokens.add(new Token(TokenType.OR, "||"));
                pos += 2;
            } else if (c == '=' && pos + 1 < input.length() && input.charAt(pos + 1) == '=') {
                tokens.add(new Token(TokenType.EQ, "=="));
                pos += 2;
            } else if (c == '!' && pos + 1 < input.length() && input.charAt(pos + 1) == '=') {
                tokens.add(new Token(TokenType.NEQ, "!="));
                pos += 2;
            } else if (c == '>' && pos + 1 < input.length() && input.charAt(pos + 1) == '=') {
                tokens.add(new Token(TokenType.GTE, ">="));
                pos += 2;
            } else if (c == '<' && pos + 1 < input.length() && input.charAt(pos + 1) == '=') {
                tokens.add(new Token(TokenType.LTE, "<="));
                pos += 2;
            } else if (c == '>') {
                tokens.add(new Token(TokenType.GT, ">"));
                pos++;
            } else if (c == '<') {
                tokens.add(new Token(TokenType.LT, "<"));
                pos++;
            } else if (Character.isLetter(c) || c == '_' || c == '$') {
                StringBuilder sb = new StringBuilder();
                while (pos < input.length() && (Character.isLetterOrDigit(input.charAt(pos)) || input.charAt(pos) == '_' || input.charAt(pos) == '$')) {
                    sb.append(input.charAt(pos));
                    pos++;
                }
                tokens.add(new Token(TokenType.VARIABLE, sb.toString()));
            } else if (Character.isDigit(c) || c == '.') {
                StringBuilder sb = new StringBuilder();
                boolean hasDot = false;
                while (pos < input.length() && (Character.isDigit(input.charAt(pos)) || input.charAt(pos) == '.')) {
                    if (input.charAt(pos) == '.') {
                        if (hasDot) {
                            throw new RuntimeException("Invalid number format");
                        }
                        hasDot = true;
                    }
                    sb.append(input.charAt(pos));
                    pos++;
                }
                tokens.add(new Token(TokenType.NUMBER, sb.toString()));
            } else {
                throw new RuntimeException("Unexpected character: " + c);
            }
        }
        tokens.add(new Token(TokenType.EOF, ""));
        return tokens;
    }
}
